/*
 * Copyright (c) 1997, Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#include "dmu.h"
#include "dmuVideoIn.h"
#include "dmuGLCtx.h"
#include <dmedia/vl_mvp.h>
#include "ScratchWindow.h"
#include <unistd.h>
#include <X11/GLw/GLwMDrawA.h>
#include <X11/keysym.h>

// bigVideoInGL.c++ - <dmu>Simple video->screen app using libdmu.  This
// is an example of a video object dumping directly to a screen
// display object.  Video can be sized by integer scale factors 
// using a GLPixel zoom.  Default zoom is 2,2.  You can set other
// integer valued zoom factors using the command line:
//
// bigVideoInGL <zoomX> <zoomY>
//
// </dmu>



// Simple macro for checking the result of a dmu method and printing
// the results if there's ana error.
#define dmuCheck(dmFunc) { \
    int dmuErrID; \
    if ((dmuErrID = (dmFunc)) != DMU_OK) {\
       fprintf(stderr,"*Error:%d:%s\n", dmuErrID, \
	       getErrorString(dmuErrID) ); \
					     /*exit(-1);*/ \
    }\
    else { \
	     /*fprintf(stderr,"No Error\n");*/ \
    } \
 }

void handleXEvents();

dmuVideoIn *videoIn;
dmuGLCtx   *context;
int sizeX,sizeY;
int zoomSizeX,zoomSizeY;
Display *dpy;

void render();
void handleDmuEvents(XtPointer *, int *fd, XtInputId*);
void handleInputEvents(Widget w, XtPointer client_data, XtPointer call);


int main(int argc, char *argv[]){
  // other variables we'll need
  fd_set readFDSet;
  int err;
  int zoomX = -1;
  int zoomY = -1;

  ScratchWindow *window;

  int optError = 0;
  extern char *optarg;
  int c;
  while ((c = getopt(argc, argv, "x:y:")) != -1) {
    switch (c) {
    case 'x':
      zoomX = atoi(optarg);
      break;
    case 'y':
      zoomY = atoi(optarg);
      break;
    default:
      optError = 1;
    }
  }

  if (optError || (zoomX == -1) || (zoomY == -1)) {
    fprintf(stderr, "Usage:%s -x <x zoom factor> -y <y zoom factor>\n",
	    argv[0]);
    exit(-1);
  }
  
  dpy = XOpenDisplay(":0");
  
  // -- Construction --
  // Create our objects.
  videoIn = new dmuVideoIn();
  videoIn->setName("VideoIn object");

  context = new dmuGLCtx();
  context->setName("Context");

  // -- Wire --
  // Wire up the path (video->context)
  videoIn->addObject(context);
  
  // -- Setup --

  //   -- Begin Video Object Setup ----

  // Tells video that we'll use the source that's set on vcp
  fprintf (stderr, "Setting up video path:\n");
  dmuCheck(videoIn->setup(VL_ANY,VL_VIDEO));

  // Tell's the video object that we want YUV pixels
  fprintf (stderr, "Setting up packing: ");
  dmuCheck(videoIn->setPacking(VL_PACKING_ABGR_8));
  dmuCheck(videoIn->setCaptureType(VL_CAPTURE_INTERLEAVED));
  dmuCheck(videoIn->setFormat(VL_FORMAT_RGB));
  dmuCheck(videoIn->setLayout(VL_LAYOUT_GRAPHICS));

  // Get the size of a single frame of video 
  videoIn->getImageDimensions(&sizeX,&sizeY);
  fprintf (stderr, "Video Image Size: %d, %d\n\n",sizeX,sizeY);


  zoomSizeX = sizeX * zoomX;
  zoomSizeY = sizeY * zoomY;

  //   -- End Video Setup ---

  //   -- Begin Context Setup --

  context->setup(dpy, sizeX, sizeY);

  //   -- End ContextSetup --

  //   -- Begin window setup --

  // Setup an Open GL window to render into - this is not a dmuObj
  window = ScratchWindow::create(dpy, zoomSizeX, zoomSizeY, &argc, argv);

  if (!window){
    fprintf (stderr, "Failed to create scratch window!\n");
    exit (-1);
  }


  // OpenGL setup for the window
  glViewport( 0, 0, zoomSizeX, zoomSizeY);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0,zoomSizeX,0,zoomSizeY,0,1000);
  glDisable(GL_DEPTH_TEST);
  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
  glPixelZoom(zoomX,zoomY);

  // -- Initialize --

  // Init only has to be called at the head of the chain.
  fprintf (stderr, "Initializing: \n");
  dmuCheck(videoIn->init());


  // -- Start --

  // Start only has to be called at the head of the chain.  The video
  // transfer has now started.
  fprintf (stderr, "Fire it up!: \n");
  dmuCheck(videoIn->start());


  window->addInput(videoIn->getFD(),(XtInputCallbackProc)handleDmuEvents,NULL);
  window->addGLCallback(GLwNinputCallback, handleInputEvents, NULL);
  window->setContext();
  window->eventLoop();

  delete (videoIn);
  delete (context);
}



void handleDmuEvents(XtPointer*, int *fd, XtInputId*){
  fd_set readFDSet;
  int maxFD;
  int error;

  FD_ZERO(&readFDSet);
  videoIn->setFD(&readFDSet, &maxFD);
  
  error = videoIn->handleEvents(readFDSet);

  // This handles the temporary videoIn return values.  We only want
  // to redraw the screen if we have a new video frame waiting for
  // us
  if (error != VIDEOIN_TRANSFER_COMPLETE){
    return;
  }

  render();
  
  // Swap buffers in the window
  glXSwapBuffers(dpy, glXGetCurrentDrawable());  

}


void handleInputEvents(Widget w, XtPointer client_data, XtPointer call){
  char buffer[31];
   KeySym keysym;

  XEvent *event = ((GLwDrawingAreaCallbackStruct *) call)->event;
  switch(event->type){
  case ButtonPress:
    exit(EXIT_SUCCESS);
    break;
  case KeyRelease:
    XLookupString(&event->xkey, buffer, 30, &keysym, NULL);
    switch(keysym) {
    case XK_Escape :
      exit(EXIT_SUCCESS);
      break;
    }
  }

}

void render(){
  int err;
	
  err = context->grab();
  if (err != DMU_OK){
    return;
  }
  //fprintf (stderr, "Grabbed\n");

  err = context->makeCurrentRead();
  //fprintf (stderr, "Current Read\n");
  
  glCopyPixels(0,0,sizeX,sizeY,GL_COLOR);
  
  context->release();
  if (err != DMU_OK){
    return;
  }
  return;
}
