/*
 * Copyright (c) 1997, Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the name of Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Silicon Graphics.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE
 * POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#include "dmu.h"
#include "dmuVideoIn.h"
#include "dmuGLTex2d.h"
#include <dmedia/vl_mvp.h>
#include "ScratchWindow.h"
#include <unistd.h>

#include <X11/GLw/GLwMDrawA.h>
#include <X11/keysym.h>

// bigVideoInTex.c++ - <dmu> Sample program displaying live video as a
// texture map, zoomed to an arbitrary floating point X and Y zoom
// factor.  If no command line options are given, then a 2.0,2.0 zoom
// is assomed. Use this command line to get other zoom factors:
//
// bigVideoInTex <zoomX> <zoomY>
//
// dmuVideoIn -> dmuGLTex2d
//
// <dmu> The textured polygon is rendered to a double-buffered Open GL
//window created through the ScratchWindow class. </dmu> Notice how
//defineTex2d is used in the handleEvents loop to define and use new
//textures for OpenGL rendering

void render(Display *theDisplay, float sizeX, float sizeY);


#define dmuCheck(dmFunc) { \
    int dmuErrID; \
    if ((dmuErrID = (dmFunc)) != DMU_OK) {\
       fprintf(stderr,"*Error:%d:%s\n", dmuErrID, \
	       getErrorString(dmuErrID) ); \
					     /*exit(-1);*/\
    }\
    else { \
	     /*fprintf(stderr,"No Error\n");*/ \
    } \
 }

int sizeX,sizeY;

void handleDmuEvents(XtPointer *, int *fd, XtInputId*);
void handleInputEvents(Widget w, XtPointer client_data, XtPointer call);

dmuVideoIn *videoIn;
dmuGLTex2d *texture;
Display *theDisplay;
float newSizeX,newSizeY;
float zoomX = -1;
float zoomY = -1;


int main(int argc, char *argv[]){
  fd_set readFDSet;
  int maxFD;
  int err;

  ScratchWindow *window;

  int optError = 0;
  extern char *optarg;
  int c;
  while ((c = getopt(argc, argv, "x:y:")) != -1) {
    switch (c) {
    case 'x':
      zoomX = atof(optarg);
      break;
    case 'y':
      zoomY = atof(optarg);
      break;
    default:
      optError = 1;
    }
  }

  if (optError || (zoomX == -1) || (zoomY == -1)) {
    fprintf(stderr, "Usage:%s -x <x zoom factor> -y <y zoom factor>\n",
	    argv[0]);
    exit(-1);
  }
    
  // Open up the X Display - We need it for the texture and scratch
  // window.
  theDisplay = XOpenDisplay( ":0" );

  // -- Construction --
  videoIn = new dmuVideoIn;
  videoIn->setName("Video In ");

  texture = new dmuGLTex2d;
  texture->setName("Texture");


  // -- Wire --
  videoIn->addObject(texture);
  

  // -- Setup --

  // Use current vcp setting for video source
  dmuCheck(videoIn->setup(VL_ANY));

  // Use ABGR_8 packing and formatting texture mapping
  //  dmuCheck(videoIn->setCaptureType(VL_TIMING_525_SQ_PIX));
  dmuCheck(videoIn->setPacking(VL_PACKING_ABGR_8));
  dmuCheck(videoIn->setCaptureType(VL_CAPTURE_INTERLEAVED));
  dmuCheck(videoIn->setFormat(VL_FORMAT_RGB));

  dmuCheck(videoIn->setSize(512,512));
  dmuCheck(videoIn->setOrigin(0,32));

  // Use graphics layout for texture maps 
  dmuCheck(videoIn->setLayout(VL_LAYOUT_GRAPHICS));

  fprintf (stderr, "Getting video dimensions: ");
  videoIn->getImageDimensions(&sizeX,&sizeY);
  fprintf (stderr, "Video dimensions: %d, %d\n\n",sizeX,sizeY);

  newSizeX = zoomX * sizeX;
  newSizeY = zoomY * (sizeY-32.0);

  // Set up the texture object to have the same size
  dmuCheck(texture->setup(theDisplay, sizeX, sizeY));
  
  // -- Init --
  fprintf (stderr, "Initializing: \n\n");
  dmuCheck(videoIn->init());

  // -- Start --
  fprintf (stderr, "Fire it up!\n");
  dmuCheck(videoIn->start());


  // -- Other setup -- 

  // Setup an Open GL window to render into - this is not a dmuObj
  window = ScratchWindow::create(theDisplay, newSizeX, newSizeY, &argc, argv);
  if (!window){
    fprintf (stderr, "Failed to create scratch window!\n");
    exit (-1);
  }
  
  // OpenGL setup for the window
  glViewport( 0, 0, newSizeX, newSizeY);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0,newSizeX,0,newSizeY,0,1000);

  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();

  glDisable(GL_DEPTH_TEST);

  glEnable(GL_TEXTURE_2D);
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
  glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE_EXT);

  window->addInput(videoIn->getFD(),(XtInputCallbackProc)handleDmuEvents,NULL);
  window->addGLCallback(GLwNinputCallback, handleInputEvents, NULL);
  window->eventLoop();

  delete (texture);
  delete (videoIn);

}


void handleDmuEvents(XtPointer*, int *fd, XtInputId*){
  fd_set readFDSet;
  int maxFD;

  FD_ZERO(&readFDSet);
  videoIn->setFD(&readFDSet, &maxFD);
  
  int error = videoIn->handleEvents(readFDSet);

  // This handles the temporary videoIn return values.  We only want
  // to redraw the screen if we have a new video frame waiting for
  // us
  if (error != VIDEOIN_TRANSFER_COMPLETE){
    return;
  }
  
  // Tell the texture to setup Open GL to use the latest
  // video frame as a texture map.
  dmuCheck(texture->defineTex2D());
  
  // Rotate and render the texture map onto a polygon
  glMatrixMode( GL_MODELVIEW );
  glPushMatrix();
  glTranslatef(newSizeX/2,newSizeY/2,0);
  render(theDisplay,newSizeX,newSizeY);
  glPopMatrix();
  
  // Swap buffers in the window
  glXSwapBuffers(theDisplay, glXGetCurrentDrawable());  

}

// Handles the clearing of the window and rendering of the polygon
void render(Display *theDisplay, float sizeX, float sizeY){
  // Clear the window
  glClearColor(0.0,0.0,0.0,0.0);
  glClear(GL_COLOR_BUFFER_BIT);
  
  glColor3f(1.0,1.0,1.0);

  glBegin(GL_QUADS);

  glTexCoord2d(1.0,1.0 - (32.0/sizeY*zoomY));
  glVertex3d(sizeX/2,sizeY/2,0.0);

  glTexCoord2d(0.0,1.0 - (32.0/sizeY*zoomY));
  glVertex3d(-sizeX/2,sizeY/2,0.0);


  glTexCoord2d(0.0,0.0);
  glVertex3d(-sizeX/2,-sizeY/2,0.0);

  glTexCoord2d(1.0,0.0);
  glVertex3d(sizeX/2,-sizeY/2,0.0);

  glEnd();
  
}  


void handleInputEvents(Widget w, XtPointer client_data, XtPointer call){
  char buffer[31];
   KeySym keysym;

  XEvent *event = ((GLwDrawingAreaCallbackStruct *) call)->event;
  switch(event->type){
  case ButtonPress:
  case KeyRelease:
    XLookupString(&event->xkey, buffer, 30, &keysym, NULL);
    switch(keysym) {
    case XK_Escape :
      exit(EXIT_SUCCESS);
      break;
    }
  }

}
