/****************************************************************************
*                                                                          *
*               Copyright (c) 1996-97-98, Silicon Graphics, Inc.           *
*                                                                          *
*   Permission to use,  copy, modify, distribute, and sell this software   *
*   and its documentation for any purpose is hereby granted without fee,   *
*   provided that the name of  Silicon  Graphics  may not be used in any   *
*   advertising  or  publicity  relating  to  the  software  without the   *
*   specific, prior written permission of Silicon Graphics.                *
*                                                                          *
*   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
*   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
*   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
*                                                                          *
*   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
*   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
*   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
*   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
*   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
*   PERFORMANCE OF THIS SOFTWARE.                                          *
*                                                                          *
****************************************************************************/

//
//  dmuChain.h
//
//  Author: Ed Allard & Tim Davison
//  Date:   Oct, 1996
//
// <dmu>
// This object takes a chain of dmuObjects and wraps them up into a single
// object.  This allows you to create another level of abstraction above
// the individual objects.
// This chain object <B>does not deal with fan-out</B> right now.  You can only
// have one drain in the chain contained by this object
// The dmuChain object can be used in a couple
// of different ways. It can be used to encapsulate sets of objects that
// are used often.  It could also be inherited from and built up to
// create diffent chains depending on a certain input.  For example, if
// you want to have a movie source, there are two objects that output movie.
// You could make a dmuChain object that would create an appropriate movie
// object depending on the movie type.  The children would only see the
// dmuChain object as the parent and the chain inside could be different
// depending on the movie type.
// </dmu>
//
#ifndef _DMUCHAIN_H
#define _DMUCHAIN_H

#include "dmuObj.h"

class dmuChain:public dmuObj {
public:
  // Constructor: creates an object that contains no chain
  dmuChain();

  // Destructor
  ~dmuChain();


  // This method is intended to be overridden to allow you to create a
  // chain of object internally for this object to contain instead of
  // passing a chain in to this object.
  // This method does not do anything on this object, but if you inherit
  // from this object, you can override this method to create a chain
  // and call setup on all of the object in the chain
  virtual int setup();
  
  // If this class is acting as a destination for another object in a
  // chain, this method is used to notify the parent object (i.e. the
  // source object) of any constraints which the head object of the chain
  // may have on its source.  These constraints are added to the <i>params</i>
  // parameter structure.  This method is used to ensure that the
  // DMbuffers that the parent object sends to this object are in a
  // format that can be received and used propperly.
  // addSrcConstraints() is ususally called internally in the library
  // during the init() method, and is rarely, if ever, used by user
  // programs.
  int addSrcConstraints(DMparams *params);

  // This method adds to the <i>params</i> structure, any constraints
  // that may be associated with the output of the tail object of the
  // chain.  Normally, this method is called internal to the
  // object to set up the parameters of a DMbuffer pool which the object
  // will use as a
  // drain.  This can be useful for child objects (or dmu programs) to
  // obtain information about the constraints used in the creation of
  // the DMbuffer pool associated with an object that drains into
  // DMbuffers.
  int addDstConstraints(DMparams *params);

  // This method call Init on the head of the chain. Because init is
  // propagated by any useable object, calling init on the head object
  // initializes the whole chain.
  int init(dmuParams *params = NULL);

  // This method addes newObj as a child of the last object in the chain.
  int addObject(dmuObj *newObj);

  // This method calls start on the head of the chain.
  int  start(int propagate = DMU_PROPAGATE);

  // this method does not have any meaning on the chain object because
  // there are potentially more than one FD within the chain.
  // Use the setFD method instead.
  int  getFD();

  // This method sets the fd in the fdSet for each object in the chain.
  int setFD(fd_set *fdSet,int *maxFD,int propagate = 1);

  // This method will pass the inBuffer to the head of the chain.
  int  receive(DMbuffer inBuffer);

  // Set the chain that this collection will contain.  
  int  setChain(dmuObj *newHead);

  // This method will go and call handle event on all of the objects in
  // the object chain.  This means that the handleEvent method for each
  // object should check to see if it's own file descriptor has been set
  // before it does anything.
  int  handleEvents(fd_set fdSet);

protected:
  dmuObj *chainHead;
  dmuObj *chainTail;

};

#endif
