/****************************************************************************
*                                                                          *
*               Copyright (c) 1996-97-98, Silicon Graphics, Inc.           *
*                                                                          *
*   Permission to use,  copy, modify, distribute, and sell this software   *
*   and its documentation for any purpose is hereby granted without fee,   *
*   provided that the name of  Silicon  Graphics  may not be used in any   *
*   advertising  or  publicity  relating  to  the  software  without the   *
*   specific, prior written permission of Silicon Graphics.                *
*                                                                          *
*   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
*   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
*   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
*                                                                          *
*   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
*   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
*   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
*   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
*   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
*   PERFORMANCE OF THIS SOFTWARE.                                          *
*                                                                          *
****************************************************************************/

//
//  dmuGLPBuffer.h
//
//  Author: Demo Factory
//  Date:   Oct, 1996
//
// <dmu>

// The dmuGLPBuffer class is intended as a base class for any
// dmu object that wants to receive and manipulate DMbuffer data as an
// Open GL Pixel Buffer.  See <i>glXCreateGLXPbufferSGIX (3G)</i> for
// more information on OGL PBuffers.  This class creates a PBuffer,
// with no pixel area associated with it. The parameters for the
// PBuffer are based on the dmu constraints obtained through the dmu
// chain initialization (the init() process - see Overview). </P>

// <P> As the parent of the object sends new frame data, the
// dmuGLPBuffer keeps a reference to the last (most recent) frame
// received.  Access to the data is obtained through the <b>grab()</b>
// method.  Grab() associates the latest frame data as the pixel area
// of the PBuffer (calls glXAssociateDMPbufferSGIX).  This PBuffer can
// now be used with any Open GL operations which are valid with
// PBuffers (for example, defining textures, reading and writing
// pixels, etc.).  The DMBuffer remains associated with the PBuffer,
// and a valid pixel store for OGL operations until <B>release()</B>
// is called on the object.  Grab() is a blocking call, so you must
// call release() after every grab() call.  Two grab calls in a row will
// cause indefinite blocking</P>

// <P> This class is intended as a base class, and should not be
// instantiated directly.  Either instantiate a class that inherits
// from this, or create your own subclass to gain access to the
// pbuffer and dmbuffers. The dmuGLTex2d object is a good example of 
// how to create your own object that inherits from dmuGLPBuffer.</P>

// </dmu>
#ifndef DMU_GLPBUFFER_H
#define DMU_GLPBUFFER_H

#include "dmuLatch.h"
#include "dmuUtil.h"

// system includes needed by this object
#include <GL/gl.h>
#include <GL/glx.h>

#define DMU_GLPBUFFER_PACKING DM_IMAGE_PACKING_RGBA
//#define DMU_GLPBUFFER_PACKING DM_IMAGE_PACKING_XRGB1555

class dmuGLPBuffer : public dmuLatch {
public:

  // Constructs and initialization of member variables.This class is
  // intended as a base class, and should not be instantiated
  // directly.
  dmuGLPBuffer();
  
  // Destroys the object, and the PBuffer that is associated with 
  // the object.
  ~dmuGLPBuffer();

  // Setup is called to set all the variables that are needed to
  // create the PBuffer and the DMbuffers that are going to be used in
  // the object.  <I>theDisplay</I> is a pointer to the X Display of
  // the application.  <I>Width and height</i> specify the size of the
  // pixel area that to be created.  This should be the same as the
  // size of a single frame of data that will be passed to it from 
  // its parent object.  Future implementations should detect
  // the size constraints automatically, but for now, they have to 
  // be set explicitly by the user process.
  int setup(Display *theDisplay, int width, int height);

  // Constructs the PBuffer based on the chain constraints and the 
  // <i>setup()</i> variables.  This is usually not called explicitly
  // unless this object is at the head of the dmu chain.  Otherwise, 
  // init() is called from the parent's init() function.
  virtual int init(dmuParams *params = NULL);

  // This method locks the latest data from the parent object, and
  // associates it as the pixel area of the PBuffer.  This sets the
  // object up to perform OpenGL operations on the PBuffer, using the
  // latest data received from the parent.  Note that <i>grab()</i> is
  // a blocking call, and should always be followed by a matching call
  // to <i>release()</i> when the object is finished with the PBuffer.
  virtual int grab();

  // If this class is acting as a destination for another object in a
  // chain, this method is used to notify the parent object (i.e. the
  // source object) of any constraints which the video path may have
  // on its source.  These constraints are added to the <i>params</i>
  // parameter structure.  addSrcConstraints() is ususally called
  // internally in the library during the init() method of the parent
  // object.  It is rarely, if ever, used by user programs.
  int addSrcConstraints(DMparams *params);

  // Currently this method is unimplemented 
  int addDstConstraints(DMparams *params);

  // returns the frame buffer config.  
  GLXFBConfigSGIX getFBConfig() {  return fbConfig;  }

protected:
  int createPBuffer();
  
  Display *dpy;
  GLXFBConfigSGIX fbConfig;
  GLXPbufferSGIX  pbuffer;
  DMparams        *gfxParams;

};

#endif
