/****************************************************************************
*                                                                          *
*               Copyright (c) 1996-97-98, Silicon Graphics, Inc.           *
*                                                                          *
*   Permission to use,  copy, modify, distribute, and sell this software   *
*   and its documentation for any purpose is hereby granted without fee,   *
*   provided that the name of  Silicon  Graphics  may not be used in any   *
*   advertising  or  publicity  relating  to  the  software  without the   *
*   specific, prior written permission of Silicon Graphics.                *
*                                                                          *
*   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
*   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
*   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
*                                                                          *
*   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
*   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
*   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
*   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
*   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
*   PERFORMANCE OF THIS SOFTWARE.                                          *
*                                                                          *
****************************************************************************/

//
//  dmuIC.h
//
//  Author: Tim Davison <td@pobox.com>
//  Date:   Oct, 1996
//
//  <dmu>
//  This object is the base class for creating objects that use the
//  special compression capabilites of the ICE chip.  This object
//  wraps up much of the digital media image converter(dmic) library.
//
//  <P>
//  ICE:  There is a chip on the O2 that does image conversion very
//  efficiently.  It has a queue of requests that it pulls from
//  as fast as it can.  With each request, you pass it a conversion
//  context.  This conversion context tells ICE what type of compression
//  or decompression to use.  Currently, ICE knows how to encode and
//  decode jpegs, decode mpeg.  This object
//  does everything you need to pass images to the ICE input queue and
//  get images from the output queue.  But this object doesn't know how
//  to create the conversion context to pass to ICE with the image.
//  The subclasses of this object implement the setup method which
//  creates the conversion context.
//  <B>IMPORTANT:</B> this object does not do anything to make sure
//   that the ICE queue has room in it for another request. </P>
//
//  This is the header file for the dmuIC object.  <B>This object can not
//  be instaniated by itself, it must be subclassed from.</B>  To use this
//  you must inherit from this object and provide an implementation of
//  the setup method.  The setup method needs to create an image conversion
//  context.
//  Here is the current hierarch of current objects derived from this
//  class.  
//  dmuObj -> dmuIC ---> dmuICjencoder
//                  |
//                   --> dmuICjdecoder
//
//
//  This is the base class for doing image conversion operations with ICE.
//  This object wraps up the similarities between the code for a decoder object
//  and the code for an encoder object.  Most things are the same in fact.
//  The only differences are in the setup method.
//  </dmu>
//
#ifndef _DMUIC_H
#define _DMUIC_H

// include the system dmedia header file for image conversion
#include <dmedia/dm_imageconvert.h>
// include the base class for this class.
#include "dmuObj.h"

class dmuIC : public dmuObj {
public:

  // <P>Constructor:  This will initialize some member variables.  The
  // defaults are as follows:</P>
  // <UL>
  // <LI> Conversion Speed: <B>DM_IC_SPEED_REALTIME</B>
  // </UL>
  // <P>If you need to change these parameters, you must change them
  // before you call setup.  Setup uses the parameters to create an
  // image conversion context that can not be changed.</P>
  // For this object to be functional, the setup method must
  // have an implementation provided.  This means that this constructor
  // should never be called except implicitly by the subclass
  dmuIC();

  // Destructor:  destroys the image conversion context if one has been
  //  allocated
  virtual ~dmuIC();

  // This is the method that creates the image conversion context.  When
  // creating the conversion context, the object has to know a couple
  // of different things.  They are shown below with defaults.
  // <TABLE BODER=1>
  // <TR><TD> Image Width</TD><TD> 0 *can set with constructor </TD></TR>
  // <TR><TD> Image Height</TD><TD> 0 *can set with constructor</TD></TR>
  // <TR><TD> Image Packing</TD><TD> DM_IMAGE_PACKING_CbYCrY</TD></TR>
  // <TR><TD>Conversion Speed</TD><TD>DM_IC_SPEED_REALTIME</TD></TR>
  // <TR><TD> Conversion Direction</TD><TD>
  //            DM_IC_CODE_DIRECTION_UNDEFINED</TD></TR>
  // </TABLE>
  // If these parameters are not right, you must change them before you
  // call this method.  The conversion context is passed to ICE with
  // every request, so that ICE knows what it is supposed to do.
  virtual int setup();

  // Creates a dmBuffer pool for the results of the image conversion
  // to be dumped into.  The pool created is constrained by the
  // output of this object.  The pool is also constrained by what
  // the children object want as their source
  int init(dmuParams *params);

  // If this class is acting as a destination for another object in a
  // chain, this method is used to notify the parent object (i.e. the
  // source object) of any constraints which the image converter may have
  // on its source.  These constraints are added to the <i>params</i>
  // parameter structure.  This method is used to ensure that the
  // DMbuffers that the parent object sends to this object are in a
  // format that can be received and used propperly.
  // addSrcConstraints() is ususally called internally in the library
  // during the init() method, and is rarely, if ever, used by user
  // programs.
  int addSrcConstraints(DMparams *params);

  // This method adds to the <i>params</i> structure, any constraints
  // that may be associated with the output of this object.  Normally,
  // this method is called internal to the object to set up the
  // parameters of a DMbuffer pool which the object will use as a
  // drain.  This can be useful for child objects (or dmu programs) to
  // obtain information about the constraints used in the creation of
  // the DMbuffer pool associated with an object that drains into
  // DMbuffers
  int addDstConstraints(DMparams *params);  

  // returns the file descriptor to select on.  This file descriptor
  // will trigger when there is output available in the output queue
  int getFD();

  // If this object's fd is set in the fdSet, the dmIC library is done
  // with the conversion of a buffer.  There is a waiting buffer to
  // propagate down to the children.  This method is normally called within
  // the event loop.  It will propagate the finished buffer down to
  // the children objects
  int handleEvents(fd_set fdSet);

  // This object's parent passes a buffer to this object by calling this
  // method with the new buffer as the argument.  This method passes the
  // buffer on to VICE to start converting it.  <P>
  // <B>Important:</B> Right now this object does not check to make
  // sure that the ICE queue has room.  
  int receive(DMbuffer inBuffer);

  // Set the speed that is required for the conversion.  Possible values
  // are: <B>DM_IC_SPEED_UNDEFINED</B>, <B>DM_IC_SPEED_REALTIME</B>, and
  // <B>DM_IC_SPEED_NONREALTIME</B>.  This must be set before you call
  // setup or the conversion context will be created with the default
  // <B>DM_IC_SPEED_REALTIME</B>.
  void setSpeed(DMicspeed _speed)           { speed = _speed;     }
  // return the speed that the conversion is currently set to run at.
  DMicspeed getSpeed()                      { return speed;       }

  // Set the way the pixels are packed coming into this object
  // The default for this is: <B>DM_IMAGE_PACKING_CbYCrY</B>.
  // For the fastest jpeg conversion, use the default.  Other packing
  // modes are much slower for jpeg conversion.
  // This must be set before you call setup or the packing will be the
  // default packing
  void setSrcPacking(DMimagepacking _packing) { srcPacking = _packing; }
  void setDstPacking(DMimagepacking _packing) { dstPacking = _packing; }
  // return the current packing mode
  DMimagepacking getSrcPacking()              { return(srcPacking);    }
  DMimagepacking getDstPacking()              { return(dstPacking);    }

  // <dmu> 
  // <B> Set the image size that the converter will use </B>
  // if you do not use the constructor of the subclass which sets the
  // initail size, you can use this function to setup the width
  // of the image we are dealing with.  If you didn't use the constructor
  // to set the initail size, you must call this method  before the
  // setup method is called or the setup method will fail
  // <UL> </dmu>
  void setImageSizeX(int _imageSizeX)      { imageSizeX = _imageSizeX; }
  void setImageSizeY(int _imageSizeY)      { imageSizeY = _imageSizeY; }
  void setImageDimensions(int _imageSizeX, int _imageSizeY) {
    imageSizeX = _imageSizeX;
    imageSizeY = _imageSizeY;
  }
  // <dmu> </UL> </dmu>

  // debugging
  // print out when you try to return an error with the DMUIC_ERR macro
  int setPrintReturnErr(int _printReturnErr);
  
protected:
  
  int               bufferSize;
  DMimageconverter  imgConvCtx;
  int               fd;
  DMicspeed         speed;
  DMcodedirection   direction;

  DMimagepacking    srcPacking;
  DMimagepacking    dstPacking;
  char             *srcCompression;
  char             *dstCompression;

  // debugging
  int               printReturnErr;
private:
};


//
//  DEBUGGING macro
//
// this macro will check the return on a statement and if it is not
// DMU_TRUE it will flag an error and print some stuff out.
// 
#define DMUIC_ERR(cond, ret) { \
    if (cond) {\
       if (printReturnErr == DMU_TRUE) { \
          fprintf(stderr, "dmuIC error:%d (line:%d file:%s) errno:%d\n", \
	          ret, __LINE__, __FILE__, errno); \
	  char errorBuf[DM_MAX_ERROR_DETAIL]; \
	  int errorNum; \
          fprintf(stderr, "Error description:%s\n", \
		  dmGetError(&errorNum, errorBuf));\
          fprintf(stderr, "   dmu errorNum:%d\n", errorNum);\
          fprintf(stderr, "   detail:%s\n", errorBuf);\
	  fprintf(stderr, "   object:%s\n", getName());\
	  fprintf(stderr, "   errno:%d\n", errno);\
       } \
       return(ret);\
    }}

#endif // _DMUIC_H
