/****************************************************************************
*                                                                          *
*               Copyright (c) 1996-97-98, Silicon Graphics, Inc.           *
*                                                                          *
*   Permission to use,  copy, modify, distribute, and sell this software   *
*   and its documentation for any purpose is hereby granted without fee,   *
*   provided that the name of  Silicon  Graphics  may not be used in any   *
*   advertising  or  publicity  relating  to  the  software  without the   *
*   specific, prior written permission of Silicon Graphics.                *
*                                                                          *
*   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
*   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
*   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
*                                                                          *
*   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
*   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
*   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
*   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
*   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
*   PERFORMANCE OF THIS SOFTWARE.                                          *
*                                                                          *
****************************************************************************/

#ifndef _DMUMOVIEIN_H
#define _DMUMOVIEIN_H

#include "dmuMovie.h"

// Dmedia stuff
#include <dmedia/dmedia.h>
#include <movie.h>

// Thread and semaphore stuff
#include <ulocks.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>


// <dmu> <P>The dmuMovieIn class is a base class for any object that
// reads frames from a movie file into a DMbuffer.  It should not be
// instantiated directly - use a class that inherits from this base
// class instead.  This class handles movie file open and setup, and
// movie file reading.  To read the frames, this object starts a new
// thread which continually attempts to read (and possibly uncompress
// and render) a frame ahead, while making sure that there is always a
// frame available for its child objects to use. The current
// implementation does not support movie audio tracks, only the image
// track</P>

// <P>The dmuMovieIn object always reads frames into DMbuffers that are
// sized to dimensions that are the nearest power of 2.  This is done
// to facilitate using the DMbuffers as texture maps.   </P>

// <P>The sequence of methods you need to call to use a dmuMovieIn
// object usually looks something like this:</P> <NL>

// <LI> Construct a movie object - The dmuMovieObject should never be
// instantiated directly.  Use a subclass like dmuRenderMovieIn.

// <LI> Set up the object to use the movie file you want associated
// with the object.  In the current implementation, once the read
// thread is started, the only way to restart it with a new movie is
// to delete the object and recreate it.

// <LI> You may want to grab the frame and/or buffer size values for
// use in setting up other objects.

//<LI> Wire into standard dmu object chain (see Overview for details)

//<LI> Start the movie read loop using the <i>start()</i> method.
//This starts up a thread which begins reading frames into memory.
//The read thread has a file descriptor associated with it which is
//used to notify the use program that a new movie frame has been read
//in and is ready for a call to handleEvents

//<LI> Handle events as discussed in the Overview section.


// <P><B>Using the movie for GL pixel operations</B> - To use the
// movie in standard (non-texturing) situations, use the
// getImageDimensions method to get the demensions of the actual frame
// data, and do all operations on the DMbuffer data from (0,0) to
// (imageSizeX,imageSizeY). </P>

// <P><B>Using the movie for texture mapping</B> - To use the movie as
// a texture map, use the <i>getBufferDimensions</i> method to get the
// size of the DMbuffer that's actally used for the frame data.  A
// number of methods are provided to make setting proper texture
// coordinates for your texture map easier.  See the <i>
// movieTexCoord2d, movieTexCoord2f, getTexCoordScaleXF and
// getTexCoordScaleYF</i> for more information. </P>

// </P>


class dmuMovieIn: public dmuMovie{
public:
  // Performs construction of object and initialization of internal
  // member variables
  dmuMovieIn();

  // Performs destruction of the object.  This closes an open movie
  // file, cleans up the associated DMbufferPool, and frees memory
  // resources.
  ~dmuMovieIn();

  // Opens the file pointed to by <i>newFileName</i> and extracts the
  // size, buffer size and other information associated with the image
  // track of the movie.  Returns DMU_OK on success, or a dmuError
  // value on failure
  virtual int setup(char *newFileName);

  // This method is purely virtual, and is implemented by any class
  // that inherits from dmuMovieIn.  Handles DMbuffer allocation, 
  // and other setup needed before the read thread starts
  virtual int init(dmuParams *params = NULL);

  // Starts the movie read thread.  This creates a separate thread
  // and starts reading the movie which has been opened by setup.
  // Once the read thread is started, it can only be stopped by 
  // destrying the object.
  virtual int start(int propagate = DMU_PROPAGATE);

  // Standard handle events - Called when the file descriptor has
  // signalled the use program that there's a new frame to be had.
  // This will pass the latest video frame down the dmu chain
  int handleEvents(fd_set fdSet);

  // Gets the file descriptor which is associated with the movie
  // read loop.  This file descriptor should be used with select
  // or poll to stall the user program until there is a new 
  // movie frame available from the read loop.
  int getFD();

  // This can only be called after <i>setup</i> and before
  // <i>init()</i>.  The default behavior of movieIn objects is to
  // render into a buffer size that is rounded up to the next power of
  // two.. This allows easy texture mapping capabilities.  There are
  // cases where This may not be appropriate (for example, when
  // sending the movie buffers directly to video out).  This method
  // allows you to override the default buffer size before the 
  // DMBuffer pool is created in init().  The new dimensions must
  // be at least as large as the movie image dimensions.  If they 
  // are not, then DMU_BAD_BUFFER_SIZE is returned.
  int setBufferDimensions(int newSizeX,int newSizeY);

  // This method calls glTexCoord2d with appropiately scaled texture
  // coordinates.  This is used to compensate for the fact that the
  // DMbuffer that is actually used to store the texture is (usually)
  // larger than the movie frame.  Call this with texture coordinates
  // that assume 0.0->1.0 mapping of the texture, and it will scale
  // them by texCoordScaleXD, and texCoordScaleYD.
  void movieTexCoord2d(GLdouble x, GLdouble y){
    glTexCoord2d(texCoordScaleXD * x, texCoordScaleYD * y);
  }

  // Same as movieTexCoord2d, except this method uses float values, 
  // and calls glTexCoord2f instead of glTexCoord2d
  void movieTexCoord2f(GLfloat x, GLfloat y){
    glTexCoord2f(texCoordScaleXF * x, texCoordScaleYF * y);
  }

  // Gets the value that is used in the methods mentioned above to
  // scale the X coordinate of a texture coordinate to compensate for
  // the size of the movie.  
  float getTexCoordScaleXF() {return (texCoordScaleXF);};

  // Gets the value that is used in the methods mentioned above to
  // scale the Y coordinate of a texture coordinate to compensate for
  // the size of the movie.  
  float getTexCoordScaleYF() {return (texCoordScaleYF);};


  // This is an internal method and shouldn't be called from a 
  // user program
  void readLoop();


  void pause();
  void resume();
  long getCurrentFrameIndex() { return (currentFrameIndex); }
  long getNumFrames() { return (numFrames); }
  void gotoFrame(int frameIndex);


  void setTimeScale(double _timeScale) {timeScale = _timeScale;}
  double getTimeScale() {return (timeScale);}

  void enableLoop()  {isLooping = 1;}
  void disableLoop() {isLooping = 0;}

protected:
  double timeScale;
  int isLooping;

  // Wrappers around the CurrentBuffer locking semaphore.
  int lockCurrentBuffer();
  int unlockCurrentBuffer();

  // dmBufferPool parameters
  DMparams *poolParams;

  // FD so we can block until there's room in the dmbuffer pool to
  // read a frame
  int poolFD;

  // Stuff for pollable semphore - This semaphore is what the user
  // process should select (or poll) on to know when there is a valid
  // currentBuffer to grab (i.e. so they know when to call
  // handleEvents())
  usema_t   *canRead;

  // Current buffer is a DMbuffer handle to the latest data that we
  // can get.  This is what should be passed on when handle events is
  // called It is locked wit a semaphore so that we are sure that
  // there is a valid buffer there when we need it (the only exception
  // is the startup case).  A lock on current buffer can be grabbed
  // and released using the lockCurrentBuffer() unlockCurrentBuffer()
  // methods.  The handledCurrent variable lets us keep track of who
  // should free currentBuffer, and who should V the pollable
  // semaphore so that the other process knows what the buffer state
  // is
  int      firstFrameHappened;
  DMbuffer currentBuffer;
  usema_t  *currentLock;
  int      handledCurrent;

  // readThread is a sproc'd thread that keeps reading new frames into
  // readBuffer, and updates currentBuffer with the latest data.  A
  // select call is used to make sure that we have room in our
  // dmbuffer pool to read the data, so this isn't spinning needlessly
  DMbuffer readBuffer;
  int readThreadFD;
  int readThreadPID;
  
  int isPlaying;

  // Movie File variables
  char      *fileName;
  MVid      movieID;
  MVid      imageTrack;
  int       numFrames;
  int       currentFrameIndex;
  double    frameRate;
  int       interlacing;
  long long durationTime;

  // Variables that will enable texture scaling
  double texCoordScaleXD;
  float  texCoordScaleXF;
  double texCoordScaleYD;
  float  texCoordScaleYF;
  
  // This method must be overridden by the child class to implement
  // the appropriate read behavior that gets a movie frame off of disk
  // and into the readBuffer.
  virtual int getImageFrameTime(long long movieTime) = 0;

  // This method must be overridden by the child class to implement
  // the appropriate read behavior that gets a movie frame off of disk
  // and into the readBuffer.
  virtual int getImageFrameIndex(int frameIndex)  = 0;


  // Variables to keep track of time and frame indicies so we know
  // what to read when.
  unsigned long long startTime;
  int lastFrameIndex;

private:
  
};

#endif
