/****************************************************************************
*                                                                          *
*               Copyright (c) 1996-97-98, Silicon Graphics, Inc.           *
*                                                                          *
*   Permission to use,  copy, modify, distribute, and sell this software   *
*   and its documentation for any purpose is hereby granted without fee,   *
*   provided that the name of  Silicon  Graphics  may not be used in any   *
*   advertising  or  publicity  relating  to  the  software  without the   *
*   specific, prior written permission of Silicon Graphics.                *
*                                                                          *
*   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
*   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
*   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
*                                                                          *
*   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
*   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
*   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
*   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
*   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
*   PERFORMANCE OF THIS SOFTWARE.                                          *
*                                                                          *
****************************************************************************/

//
//  dmuObj.h
//
//  Author: Ed Allard & Tim Davison
//  Date:   Oct, 1996
//
// This is the base object for the entire framework.  Every object in
// the framework inherits from this object.  The most important part of
// this object is the support for handling children objects.  Each object
// that inherits from this object has the ablility to have a list of children
// objects that it sends data down to.  This framework essentially creates a
// DAG (directed acyclic graph) which describes where data should flow.
// For example, video in could flow to a display object and also to an
// an encoder object.  So the video in object has two children, the
// encoder and the display object.  Then the encoder could have children also.
// From the encoder, the video in is dumped to disk.  This means that the
// encoder has one child, a "write to disk" object.  The graph would look
// like this:
//
//   Video in  -------->  Display
//                 |
//                 ---->  Encoder ----> Disk
//
// This dmuObj class has the support for creating these graphs of objects.
// It also know how to pass data to children.
//  Important Note: Right now there is no support for modifying the list of
//    children of an object.  So once the graph is created, it can't be
//    modified.
//    Allowing arbitrary modifications of the graph will probably never
//    be supported because of the way the dmbuffer creation works.
//    The creation of a dmbuffer depends on all the different things that
//    touch it.  For example, if you create a dmbuffer to deal with video in
//    the setup of the dmbuffer has to change when you want to use the
//    graphics subsytem with that same dmbuffer.
//    Also, the init method will call the init method on all of the
//    children, so you need to have the graph created and then call
//    init on the root, and the initialization will propagate down for you.
//
// The way it works:
//   When you use this framework that are a couple of things you have to do.
//   You must create the graph of all of the objects before you call the
//   init method.  The init method creates a dmbuffer in each parent object.
//   When the parent creates the dmbuffer within itself, the parent uses
//   its own constraints in addition to those of its children.  So if the
//   graph is not done before the init method, the dmbuffers will be created
//   with the wrong parameters.  This will cause lots of problems.
//   Each object in this system supports two special methods that it
//   inherits from this object.  The methods are:
//     addSrcConstraints
//     addDstConstraints
//   These methods take a DMparams structure and add constraints to them.
//   The constraints these methods add are the ones that determine how a
//   dmbuffer should be created.
//   AddSrcConstraints add the constraints for the source of the object
//     which is implementing the method. ie the input of the object
//   AddDstConstraints add the constraints for the destination of the object
//     which is implementing the method. ie the output of the object
//   Example:  When you call AddSrcConstraints on an encoder object, it will
//     add the constraints for what the encoder wants to encode.  In other
//     words it source constraints define what the raw (uncompressed) data
//     looks like.  Destination constraints deal with the encoders output.
//
//   This means that when a parent object creates its dmbuffer, it calls
//   addSrcConstraints on all of this children.  This will add all of their
//   constraints to the list of constraints.  Then the parent will call
//   addDstConstraints on itself to get the constraints it places on
//   the dmbuffer.  The parent's dmbuffer is where the parent places
//   its data when it's done.  So it's the destination or output of the
//   parent.  After adding all of the constraints, the parent can then
//   try to create a dmbuffer which meets all of the constraints.  
//
#ifndef _DMUOBJ_H
#define _DMUOBJ_H

// include some dmedia header files 
#include <dmedia/dm_params.h>
#include <dmedia/dm_buffer.h>
#include "dmu.h"
#include "dmuUtil.h"


// Thread and semaphore stuff
#include <ulocks.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

// Forward declarations
class dmuBuffer;
class dmuObj;

// This is a place holder for future use in initialization
typedef struct _dmuParams {
  int foo;// junk for now.. 
}dmuParams;

// This is the data structure that holds the list of children
typedef struct _dmuObjList {
  dmuObj *obj;
  _dmuObjList *next;
} dmuObjList;

// a couple of utility functions
extern const char *getErrorString(int errorID);
extern void oldPrintDMParams(const DMparams *params);
extern void printDMParams(const DMparams *params);


class dmuObj {
public:
  // Constructor: Does some initialization
  dmuObj();
  // Destructor:
  virtual ~dmuObj();

  // This method will add "newObj" to the list of children
  // NOTE: As of right now, there is no support for modifying the list
  // of children.  
  virtual int addObject(dmuObj *newObj);

  // <dmu>
  // These methods are explained a bit above.
  // NOTE: Both of these methods should be impelemented by the subclass of
  // this objects, so if either of these methods are called, they return
  // an error <UL> </dmu>
  virtual int addSrcConstraints(DMparams *params) {return DMU_VIRTUAL_CALLED;}
  virtual int addDstConstraints(DMparams *params) {return DMU_VIRTUAL_CALLED;}
  //<dmu> </UL> </dmu>
  
  // This is the method that initializes the whole system
  // It causes each parent object to create a dmbuffer. 
  virtual int init(dmuParams *params = NULL);


  // this is another method that will propagate down the graph
  // this is to be used to start stuff up
  virtual int start(int propagate = DMU_PROPAGATE);


  // This is a method to stop the chain from processing.  If new 
  // buffers come it, they will be ignored.  If I'm an object
  // that feeds others, then I stop feeding.
  virtual int stop();

  // This method should return a file desciptor you can select on
  // which will signal when you have new data ready to propagate to
  // you children
  virtual int getFD();

  // This method will set up the FD set for you for the _entire chain_
  // It will propagate down, calling setFD on all it the children
  virtual int setFD(fd_set *fdSet,int *maxFD, int propagate = 1);

  // This is the method that will handle stuff when the fd is signaled.
  // NOTE: see above note about addSrcContstraints 
  virtual int handleEvents(fd_set fdSet) { return DMU_VIRTUAL_CALLED; }
  // This is the method that the parents calls on the children when the
  // parents passes data on to the parent
  // NOTE: see above note about addSrcContstraints 
  virtual int receive(DMbuffer inBuffer) { return DMU_VIRTUAL_CALLED; }

  // This allows the children to have a pointer back to the parent.
  void setParent(dmuObj *newParent)     { parent = newParent; }

  // Allows you to set the number of buffers that will be in the buffer pool
  void setNumBuffers(int newNumBuffers) { numBuffers = newNumBuffers; }
  // returns the number of buffers in the pool
  int getNumBuffers()                   { return numBuffers; }

  // GetImageDimensions sets the values pointed to by
  // <i>returnWidth</i> and <i>returnheight</i> to the size of one
  // frame of image data that is associated with the dmu object.
  // <i>Note that this may not be the same size as the DMbuffers that
  // are passed down the chain to/from the object see
  // getBufferDimensions</i>
  void getImageDimensions(int *returnWidth, int *returnHeight);


  // GetBufferDimentsions sets the values pointed to by
  // <i>returnWidth</i> and <i>returnheight</i> to the size of the
  // DMBuffer that is associated with the one frame of data for the
  // dmu object.  <i>Note that this may not be the same size as the
  // frame sizes that are being used in the movie.</i> The buffer may
  // be a different size if we want to be using the dmbuffers for
  // texture maps, for example.  Buffer Dimensions are measured in
  // pixels, not bytes, as different packing modes effect the number
  // of bytes.
  void  getBufferDimensions(int *sizeX,int *sizeY);


  // <dmu> 
  // <LI> <B>Convenience Methods</B> for geting width and height of the
  // image data and DMBuffers associated with the data (the individual
  // parts of what getImageDimensions and getBufferDimensions
  // returns).  <UL>
  //</dmu>

  int getImageSizeX() {return imageSizeX;}
  int getImageSizeY() {return imageSizeY;}

  int getBufferSizeX() {return bufferSizeX;}
  int getBufferSizeY() {return bufferSizeY;}

  //<dmu> </UL> </dmu>

  // Mostly for debugging purposes.  Give the object a name
  void setName(const char *newName);
  const char *getName()                 { return name; }

  dmuObjList *getObjectList() {return (objList);}

protected:

  // This is the method that know how to send a dmbuffer down to all of
  // the object's children
  virtual int propagate(DMbuffer outBuffer);


  // This method will add the constraints of all of the children
  int addChildConstraints(DMparams *params);
  int addParentConstraints(DMparams *params);

  // This should be overridden and return TRUE if the object 
  // is easy to add as a child at run-time.
  virtual int easyToAdd() {return DMU_FALSE;};

  // Check to see if we've been started yet
  virtual int isStarted() {return (started);}

  virtual int isInitialized() {return (initialized);}

  // member variable to store a pointer to the parent object
  dmuObj     *parent;
  // pointer to the head of the list of children
  dmuObjList *objList;
  // pointer to a DMU buffer
  //  Not used right now
  dmuBuffer  *buffer;
  // member to hold the dmbuffer pool
  DMbufferpool bufferPool;

  // Semaphore arenat that we're all gonna use if we need it
  static usptr_t   *dmuArena;

  // number of buffers in the pool
  int numBuffers;


  // These sizes are in _pixels_ not bytes, since bytes depend on
  // pixel packings.
  int imageSizeX;
  int imageSizeY;
  int bufferSizeX;
  int bufferSizeY;


  // state vars
  int  started;
  int  initialized;

  
private:

  // name of the object - debugging
  char *name;

};

#endif // _DMUOBJ_H
