/****************************************************************************
*                                                                          *
*               Copyright (c) 1996-97-98, Silicon Graphics, Inc.           *
*                                                                          *
*   Permission to use,  copy, modify, distribute, and sell this software   *
*   and its documentation for any purpose is hereby granted without fee,   *
*   provided that the name of  Silicon  Graphics  may not be used in any   *
*   advertising  or  publicity  relating  to  the  software  without the   *
*   specific, prior written permission of Silicon Graphics.                *
*                                                                          *
*   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
*   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
*   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
*                                                                          *
*   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
*   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
*   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
*   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
*   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
*   PERFORMANCE OF THIS SOFTWARE.                                          *
*                                                                          *
****************************************************************************/

#ifndef _DMUVIDEO_H
#define _DMUVIDEO_H

#include "dmuObj.h"
#include <dmedia/vl.h>
#include <dmedia/vl_mvp.h>

// dmuVideo - base class for video I/O 
//

// <dmu> <P>The dmuVideo class acts as a base class for video I/O in
// libdmu.  This class handles functions that are associated with the
// VL video server and a single VL video path.  The path has one
// source node and one drain node (<i>man vlintro</i> for more
// information on VL nodes and paths).  This object also maintains VL
// control values such as frame size, zoom factor, packing mode,
// etc.. Most of these controls can be set and queried using methods
// outlined below.  In the current implementation, these controls
// values must be set <i>before</i> the video transfer begins.  Later
// versions should allow run-time control over these variables</P>

// <P>This object handles the starting and stopping of the video transfer
// along the path.  The <i>start</i> method begins the transfer, and
// destruction of the object ends the transfer.  Later implementations
// will allow the transfer to be interrupted and restarted while it is
// running</P>

// <P>Video objects should be created and used in the following manner:
// <NL> 

// <LI> Construct a video object - dmuVideo should never be
// instantiated directly.  Use a subclass like dmuVideoIn 

//<LI> Create the video path using the <i>setup()</i> method 

//<LI> Set any non-default control values (using set methods). For
//example, the video size should be set to crop and/or zoom to a
//texture-compatible size if the video will be used as a texture map.
//<i>Note - There are known problems with capturing video into
//graphics-mode dmbuffers that are non-square in size.  The best
//parameters for texture mapping are: 512x512, with the video origin
//set to 32.</i>

//<LI> Wire into standard dmu object chain (see Overview for details)

//<LI> Start the video transfer using the <i>start()</i> method.

//<LI> Handle events as discussed in the Overview section.

// </P>The dmuVideo object also maintains an open connection to the
//video server.  This connection is a static method for dmuVideo, and
//is opened upon the first construction of a dmuVideo object (or any
//object that inherits from the dmuVideo), and closed when all
//dmuVideo objects are destroyed.  The server is shared by all
//dmuVideo objects in the same process. Users of any dmuVideo class do
//not need to worry about opening or closing video server
//connections.</dmu>

// Control value defaults
#define VIDEOIN_DEFAULT_TIMING       VL_TIMING_525_SQ_PIX
#define VIDEOIN_DEFAULT_PACKING      VL_PACKING_ABGR_8
#define VIDEOIN_DEFAULT_CAP_TYPE     VL_CAPTURE_INTERLEAVED
#define VIDEOIN_DEFAULT_RATE         30
#define VIDEOIN_DEFAULT_FORMAT       VL_FORMAT_RGB
#define VIDEOIN_DEFAULT_LAYOUT       VL_LAYOUT_LINEAR
#define VIDEOIN_DEFAULT_ZOOM_NUM     1
#define VIDEOIN_DEFAULT_ZOOM_DENOM   1
#define VIDEOIN_DEFAULT_OFFSET_X     0
#define VIDEOIN_DEFAULT_OFFSET_Y     0



class dmuVideo:public dmuObj {
public:

  static int getTypeByName(const char *deviceName);

protected:
  // Wrappers around VL functions
  int setIntControl   (VLPath path, VLNode node, int control, int value);
  int setFractControl (VLPath path, VLNode node, int control, 
		       int num, int denom);
  int setXYControl (VLPath path, VLNode node, int control, 
		       int x, int y);

  int getXYControl (VLPath path, VLNode node,int control, 
		    int *returnX, int *returnY);
  
  // The video server only has to open once. We keep track of the 
  // number of objects that are created as a static variable so
  // we can only keep one server open.
  static VLServer server;
  static int videoOpen;
  static int nObjects;

  static int checkVideoServer();
  static int openVideo();
  static int closeVideo();

  // A path will always have 2 (at least) nodes and a path between them.
  VLNode drainNode;
  VLNode sourceNode;
  VLPath path;

  int transferStarted;

public:
  // Construct a dmuVideo object.  The constructor initializes member
  // variables.  If this is the first dmuVideo object opened, the
  // constructor opens a connection the video server for all dmuVideo
  // objects to use.
  dmuVideo();

  // Destructor.  Cleans up the video path.  If this is the last
  // dmuVideo object in the process, then it closes the connection to
  // the video server.
  ~dmuVideo();

  // Create the video source and drain nodes, and a path connecting them.
  // this is a pure virtual function that is implemented by any class that
  // inherits from dmuVideo.
  int setup();


  // Setup the path - The method is purely virtual.  It will set up
  // the video path for so the transfer can begin.  This method also
  // allocates any DMbuffer pool which may be needed for the the video
  // transfer.  The <I>params</I> variable is currently ignored. 
  // Note that a calling init() will result in init() being called on 
  // all of this object's children as well
  virtual int init(dmuParams *params = NULL);
  
  // If this class is acting as a destination for another object in a
  // chain, this method is used to notify the parent object (i.e. the
  // source object) of any constraints which the video path may have
  // on its source.  These constraints are added to the <i>params</i>
  // parameter structure.  This method is used to ensure that the
  // DMbuffers that the parent object sends to this object are in a
  // format that can be received and used propperly.
  // addSrcConstraints() is ususally called internally in the library
  // during the init() method, and is rarely, if ever, used by user
  // programs.
  virtual int addSrcConstraints(DMparams *params);

  // This method adds to the <i>params</i> structure, any constraints
  // that may be associated with the output of this object.  Normally,
  // this method is called internal to the object to set up the
  // parameters of a DMbuffer pool which the object will use as a
  // drain.  This can be useful for child objects (or dmu programs) to
  // obtain information about the constraints used in the creation of
  // the DMbuffer pool associated with an object that drains into
  // DMbuffers
  int addDstConstraints(DMparams *params);

  // Starts the video transfer and prepare to handle video events
  // and/or receive buffers.  This must be called after the video
  // object is set up (<i>setup()</i>), initialized (<i>init()</i>),
  // and had all of it's control values set.  In the current
  // implementation, once the transfer is started, the only way to
  // stop the transfer it to destroy the object.
  int start(int propagate = DMU_PROPAGATE);

  // Gets the file descriptor associated with the video path.  This
  // allows the user to use <i>select(2)</i> or <i>poll(2)</i> to
  // block on this file descriptor until there is a video event that
  // should be handled.  See the Overview for details on the event
  // loop and handle events
  int getFD();

  // This is a purely virtual method that is called when the parent
  // object passes us a new DMbuffer.  See the Overview section and
  // dmuObj pages for more information on how receive works.
  virtual int receive(DMbuffer inBuffer);

  // This method is called by a user program when it's ready for us to
  // handle an event associated with our video path.  This is purely
  // virtual, and should be implemented by the inheriting class.
  virtual int handleEvents(fd_set fdSet);

  // Return the size of a buffer which would be used for a single
  // video transfer (either a full frame, or a field, depending on the
  // capture mode).  This is measured in bytes, not pixels as in
  // buffer dimensions.
  int getBufferBytes(int *returnSize);

protected:

private:
};

#endif
