/****************************************************************************
*                                                                          *
*               Copyright (c) 1996-97-98, Silicon Graphics, Inc.           *
*                                                                          *
*   Permission to use,  copy, modify, distribute, and sell this software   *
*   and its documentation for any purpose is hereby granted without fee,   *
*   provided that the name of  Silicon  Graphics  may not be used in any   *
*   advertising  or  publicity  relating  to  the  software  without the   *
*   specific, prior written permission of Silicon Graphics.                *
*                                                                          *
*   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
*   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
*   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
*                                                                          *
*   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
*   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
*   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
*   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
*   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
*   PERFORMANCE OF THIS SOFTWARE.                                          *
*                                                                          *
****************************************************************************/

#ifndef _DMUVIDEOIN_H
#define _DMUVIDEOIN_H

#include "dmuVideo.h"

// dmuVideoIn - Live video input from any video port into a DMbuffer Pool

// <dmu> <P> The dmuVideoIn class creates a video path from a single
// video input source (on an O2 with the pro-video option, this is
// either camera, composite video, or s-video input) into memory.
// After the transfer has been started, video events from the VL come
// into the object and trigger the file descriptor associated with the
// path to notify the user program.  The user program should then call
// the handleEvents method on the dmuVideo object.  If the video event
// is signaling the succesful capture of a full video frame (or field,
// depending on the CaptureType control value), then the video frame
// associated with the event is pulled into a DMbuffer, and propagated
// down the dmu chain.  See dmuVideo for more information on control
// values and setup procedures for a dmuVideo class
// <i>Note - There are known problems with capturing video into
// graphics-mode dmbuffers that are non-square in size.  The best
// parameters for texture mapping are: 512x512, with the video origin
// set to 32.</i> </dmu>

class dmuVideoIn : public dmuVideo{
public:

  // The constructor performs the normal construction of a dmuVideo object.
  dmuVideoIn();

  // The destructor performs the norman destruction of a dmuVideo object
  ~dmuVideoIn();

  // The setup method creates the actual VL path for the object.  The
  // source parameter must be one of: VL_ANY,
  // VL_MVP_VIDEO_SOURCE_CAMERA, VL_MVP_VIDEO_SOURCE_COMPOSITE, or
  // VL_MVP_VIDEO_SOURCE_SVIDEO.  VL_ANY, which is the default value,
  // uses the default video source as set in the video control panel
  // <i>(vcp (1))</i>.  In the current implementation, the srcClass
  // must be VL_VIDEO.  
  int setup(int source = VL_ANY,int srcClass = VL_VIDEO);

  // This is a convenience method which will set all the video
  // controls to the default values. See the header file or the 
  // dmuVideo documentation for more information.
  int setDefaultControls();
  
  // Init sets up the DMbuffer pool that will be used as the drain for
  // the video transfer.  This should be called after the entire dmu
  // chain has been set up, because this method obtains the
  // constraints of all of it's child objects before creating the
  // buffer pool. See the Overview section for more information on
  // init().
  int init(dmuParams *params = NULL);

  // <P>If <i>fd_set</i> has the file descriptor for the video path set,
  // then handleEvents will receive the next event on the VL video
  // queue and process it.  If this event is a VLTransferComplete
  // event, signaling that a frame is ready, then the new frame is
  // pulled into the DMbuffer pool, and all of the child objects are
  // notified.  
  // </P>

  // <P> <B>Return Values:</B> dmuVideoIn uses a different convention
  // for return values than most objects in libDMU.  The current
  // implementation returns either VIDEOIN_RANDOM_EVENT or
  // VIDEOIN_TRANSFER_COMPLETE if a video event was received
  // sucessfully, depending on what video event was received.  These
  // two values are used to make debugging problems with the video
  // driver easier, as it sometimes happens that VLTransferComplete
  // events aren't caught.  If a program notices that a dmuVideoIn
  // object is not receiving VLTranferComplete events, but is
  // receiving a number of other video events, it may want to restart
  // the tranfer (by deleting the dmuVideoIn object and recreating
  // it).  Once the problem is tracked down, the object will only
  // receive TransferComplete events.  If an error occurs in handling
  // an event, or if there is no valid event in the path's event
  // queue, then an error value is returned (see dmuError.h)
  int handleEvents(fd_set fdSet);
protected:

  // Pool parameters used for creating the dmbuffer pool.
  DMparams* poolParams;
  int xfersize;
private:
public:
  
public:
  // <dmu>

  // <LI><H3>Control Value Methods</H3> These methods set and get
  // control values on the video path.  For now, you'll have to set
  // these controls <i>after</i> doing a <i>setup ()</i> and
  // </i>before</i> doing an <i>init()</i> and <i>start()</i> See
  // <i>man vlGetControl (3)</i> for more information on the controls
  // listed here.<UL>

  // </dmu>

  // The default Zoom factor is 1/1
  int setZoom(int num, int denom) {
    return (setFractControl (path, drainNode, VL_ZOOM, num, denom));
  }

  // ZoomSize is a control that works only on composite and s-video
  // sources.  This control zooms the associated video down to the
  // size specified by <i>x</i> and <i>y</i>.  Note that this may
  // not maintain the original aspect ratio of the video.  This 
  // control is particularly useful when texture mapping video, since
  // texture map dimensions have to be power of 2.
  int setZoomSize(int x, int y) {
    return (setXYControl (path, drainNode, VL_MVP_ZOOMSIZE, x, y));
  }

  // The default packing mode is VL_PACKING_ABGR_8.  
  int setPacking(int packing){
    return (setIntControl (path, drainNode, VL_PACKING, packing));
  }

  // Default timing is VL_TIMING_525_SQ_PIX
  int setTiming(int timing){
    return (setIntControl (path, drainNode, VL_TIMING, timing));
  }

  // Default format is: VL_FORMAT_RGB. 
  int setFormat(int format){
    return (setIntControl (path, drainNode, VL_FORMAT, format));
  }

  // Default capture mode is VL_CAPTURE_INTERLEAVED.
  int setCaptureType(int capType){
    return (setIntControl (path, drainNode, VL_CAP_TYPE, capType));
  }

  // Not currently used
  int setRate(int num, int denom){
    return (setFractControl(path, drainNode, VL_RATE, num, denom));
  }

  // Default layout is VL_LAYOUT_LINEAR.  Use this layout if you
  // will be mapping the data, or performing any computation on
  // the buffer.  Set ths value to VL_LAYOUT_GRAPHICS when dealing
  // with GL objects in the chain (eg texture maps), as this mode
  // is much faster (and sometimes required) when using Open GL
  int setLayout(int layout){
    return (setIntControl (path, drainNode, VL_LAYOUT, layout));
  }

  // Default size is the full, un-zoomed size of the video source
  int setSize(int width, int height){
    return (setXYControl (path, drainNode, VL_SIZE, width, height));
  }

  // Default offset is (0,0)
  int setOffset(int x, int y){
    return (setXYControl (path, drainNode, VL_OFFSET, x, y));
  }

  int setOrigin(int x, int y){
    return (setXYControl (path, sourceNode, VL_ORIGIN, x,y));
  }

  // We override the default behaviors on these, so we can query the
  // video path directly

  // Return the size of a 'frame' which could be full frames, or could
  // be fields, depending on the capture mode.
  void getImageDimensions(int *returnWidth, int *returnHeight){
    getXYControl (path, drainNode, VL_SIZE, returnWidth, returnHeight);
  }

  // Return the size of a 'frame' which could be full frames, or could
  // be fields, depending on the capture mode.
  int getImageSizeX(){
    int returnWidth,height;
    getXYControl (path, drainNode, VL_SIZE, &returnWidth, &height);
    return (returnWidth);
  }

  int getImageSizeY(){
    int width,returnHeight;
    getXYControl (path, drainNode, VL_SIZE, &width, &returnHeight);
    return (returnHeight);
  }

  // Return the size of a 'frame' which could be full frames, or could
  // be fields, depending on the capture mode.
  void getBufferDimensions(int *returnWidth, int *returnHeight){
    getXYControl (path, drainNode, VL_SIZE, returnWidth, returnHeight);
  }

  // Return the size of a 'frame' which could be full frames, or could
  // be fields, depending on the capture mode.
  int getBufferSizeX(){
    int returnWidth,height;
    getXYControl (path, drainNode, VL_SIZE, &returnWidth, &height);
    return (returnWidth);
  }

  int getBufferSizeY(){
    int width,returnHeight;
    getXYControl (path, drainNode, VL_SIZE, &width, &returnHeight);
    return (returnHeight);
  }





  // <dmu> </UL> </dmu>  
};

#endif
