/****************************************************************************
*                                                                          *
*               Copyright (c) 1996-97-98, Silicon Graphics, Inc.           *
*                                                                          *
*   Permission to use,  copy, modify, distribute, and sell this software   *
*   and its documentation for any purpose is hereby granted without fee,   *
*   provided that the name of  Silicon  Graphics  may not be used in any   *
*   advertising  or  publicity  relating  to  the  software  without the   *
*   specific, prior written permission of Silicon Graphics.                *
*                                                                          *
*   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
*   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
*   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
*                                                                          *
*   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
*   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
*   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
*   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
*   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
*   PERFORMANCE OF THIS SOFTWARE.                                          *
*                                                                          *
****************************************************************************/

#ifndef DMU_VIDEOOUT_H
#define DMU_VIDEOOUT_H

#include "dmuVideo.h"

// <dmu> The dmuVideoOut object sends dmBuffers it receives from its
// parent object to video output.  As buffers come in, the videoOut
// object keeps a handle to the latest buffer.  The actual transfer to
// the video output device happens in the handleEvents routine which
// is called from the user program after a select call on the object's
// file descriptor falls through.  This ensures that the video output
// queue is not filled faster than the output device can handle.
// </dmu>

class dmuVideoOut:public dmuVideo{
public:
  // Constructs and initializes a dmuVideoOut object.  No VL path
  // construction or memory allocation happens here - That happens
  // in setup()
  dmuVideoOut();


  // The destructor destroys our video path, and shuts down the video
  // server (this behavior is inherited from dmuVideo).
  ~dmuVideoOut();
  
  // Setup creates the VL memory->video path for the object.
  int setup(int destDevice = VL_ANY);

  // This is a convenience method which will set all the video
  // controls to the default values. See the header file or the 
  // dmuVideo documentation for more information.
  int setDefaultControls();
  
  // Init sets up the pointers and locks that will be used to keep
  // track of buffers used for the video transfer.  This should be
  // called after the entire dmu chain has been set up, because this
  // method obtains the constraints of all of it's child objects
  // before creating the buffer pool. See the Overview section for
  // more information on init().
  int init(dmuParams *params = NULL);


  // Handles dmbuffers that come down from the parent object.  This
  // method is for internal library use, and shouldn't be called by
  // the user program.
  int receive(DMbuffer inBuffer);

  // Handles the VL events for the video path.  This gets called from
  // the user program's main handle events loop whenever the select()
  // call on the videoOut objects file descriptor falls through.
  // HandleEvents will trigger the transfer of the most recent
  // DMBuffer that it has received from memory to the video output
  // device.
  int handleEvents(fd_set fdSet);

  // For internal use only. Called by the parent objects init() method.
  int addSrcConstraints(DMparams *params);
protected:

  // We need to be able to lock our buffer handle.  newBuffer is a
  // pointer to the latest data that we've got from our parent object.
  usema_t *newLock;

  // Here are the handles to the buffers mentioned above.
  DMbuffer newBuffer;

  int gotFirstFrame;

  // Convenience method to send the currentBuffer to videoOut (since
  // we need to do this in at least 2 places)
  int sendCurrentBuffer();
  
  // methods to lock them
  void lockNewBuffer();
  void unlockNewBuffer();

  void lockCurrentBuffer();
  void unlockCurrentBuffer();

  
public:
  // <dmu>

  // <LI><H3>Control Value Methods</H3> These methods set and get
  // control values on the video path.  For now, you'll have to set
  // these controls <i>after</i> doing a <i>setup ()</i> and
  // </i>before</i> doing an <i>init()</i> and <i>start()</i> See
  // <i>man vlGetControl (3)</i> for more information on the controls
  // listed here.<UL>

  // </dmu>

  // The default Zoom factor is 1/1
  int setZoom(int num, int denom) {
    return (setFractControl (path, sourceNode, VL_ZOOM, num, denom));
  }

  // ZoomSize is a control that works only on composite and s-video
  // sources.  This control zooms the associated video down to the
  // size specified by <i>x</i> and <i>y</i>.  Note that this may
  // not maintain the original aspect ratio of the video.  This 
  // control is particularly useful when texture mapping video, since
  // texture map dimensions have to be power of 2.
  int setZoomSize(int x, int y) {
    return (setXYControl (path, sourceNode, VL_MVP_ZOOMSIZE, x, y));
  }

  // The default packing mode is VL_PACKING_ABGR_8.  
  int setPacking(int packing){
    return (setIntControl (path, sourceNode, VL_PACKING, packing));
  }

  // Default timing is VL_TIMING_525_SQ_PIX
  int setTiming(int timing){
    return (setIntControl (path, sourceNode, VL_TIMING, timing));
  }

  // Default format is: VL_FORMAT_RGB. 
  int setFormat(int format){
    return (setIntControl (path, sourceNode, VL_FORMAT, format));
  }

  // Default capture mode is VL_CAPTURE_INTERLEAVED.
  int setCaptureType(int capType){
    return (setIntControl (path, sourceNode, VL_CAP_TYPE, capType));
  }

  // Not currently used
  int setRate(int num, int denom){
    return (setFractControl(path, sourceNode, VL_RATE, num, denom));
  }

  // Default layout is VL_LAYOUT_LINEAR.  Use this layout if you
  // will be mapping the data, or performing any computation on
  // the buffer.  Set ths value to VL_LAYOUT_GRAPHICS when dealing
  // with GL objects in the chain (eg texture maps), as this mode
  // is much faster (and sometimes required) when using Open GL
  int setLayout(int layout){
    return (setIntControl (path, sourceNode, VL_LAYOUT, layout));
  }

  // Default size is the full, un-zoomed size of the video source
  int setSize(int width, int height){
    return (setXYControl (path, sourceNode, VL_SIZE, width, height));
  }

  // Default offset it (0,0)
  int setOffset(int x, int y){
    return (setXYControl (path, sourceNode, VL_OFFSET, x, y));
  }


  // We override the default behaviors on these, so we can query the
  // video path directly

  // Return the size of a 'frame' which could be full frames, or could
  // be fields, depending on the capture mode.
  void getImageDimensions(int *returnWidth, int *returnHeight){
    getXYControl (path, sourceNode, VL_SIZE, returnWidth, returnHeight);
  }

  // Return the size of a 'frame' which could be full frames, or could
  // be fields, depending on the capture mode.
  int getImageSizeX(){
    int returnWidth,height;
    getXYControl (path, sourceNode, VL_SIZE, &returnWidth, &height);
    return (returnWidth);
  }

  int getImageSizeY(){
    int width,returnHeight;
    getXYControl (path, sourceNode, VL_SIZE, &width, &returnHeight);
    return (returnHeight);
  }

  // Return the size of a 'frame' which could be full frames, or could
  // be fields, depending on the capture mode.
  void getBufferDimensions(int *returnWidth, int *returnHeight){
    getXYControl (path, sourceNode, VL_SIZE, returnWidth, returnHeight);
  }

  // Return the size of a 'frame' which could be full frames, or could
  // be fields, depending on the capture mode.
  int getBufferSizeX(){
    int returnWidth,height;
    getXYControl (path, sourceNode, VL_SIZE, &returnWidth, &height);
    return (returnWidth);
  }

  int getBufferSizeY(){
    int width,returnHeight;
    getXYControl (path, sourceNode, VL_SIZE, &width, &returnHeight);
    return (returnHeight);
  }



  // Return the size of a buffer which would be used for a single
  // video transfer (either a full frame, or a field, depending on the
  // capture mode).  This is measured in bytes, not pixels as in
  // buffer dimensions.
  int getBufferBytes(int *returnSize);


  // <dmu> </UL> </dmu>

};

#endif
