/****************************************************************************
*                                                                          *
*               Copyright (c) 1996-97-98, Silicon Graphics, Inc.           *
*                                                                          *
*   Permission to use,  copy, modify, distribute, and sell this software   *
*   and its documentation for any purpose is hereby granted without fee,   *
*   provided that the name of  Silicon  Graphics  may not be used in any   *
*   advertising  or  publicity  relating  to  the  software  without the   *
*   specific, prior written permission of Silicon Graphics.                *
*                                                                          *
*   THIS SOFTWARE IS PROVIDED "AS-IS"  AND WITHOUT WARRANTY OF ANY KIND,   *
*   EXPRESS,  IMPLIED OR OTHERWISE,  INCLUDING  WITHOUT LIMITATION,  ANY   *
*   WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.       *
*                                                                          *
*   IN  NO  EVENT  SHALL  SILICON  GRAPHICS  BE  LIABLE FOR ANY SPECIAL,   *
*   INCIDENTAL,  INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,  OR  ANY   *
*   DAMAGES  WHATSOEVER  RESULTING  FROM  LOSS OF USE,  DATA OR PROFITS,   *
*   WHETHER OR NOT ADVISED OF  THE  POSSIBILITY  OF  DAMAGE,  AND ON ANY   *
*   THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR   *
*   PERFORMANCE OF THIS SOFTWARE.                                          *
*                                                                          *
****************************************************************************/



#include "dmu.h"
#include "dmuVideoIn.h"
#include "dmuVideoOut.h"
#include "dmuGLDisplay.h"
#include <dmedia/vl_mvp.h>
#include <dmedia/vl.h>

// videoInOut.c++ - <dmu>Brings video in and displays it to both
// screen and video output using libdmu.  This demostrates fan-out in
// dmu chains and provides a simple example of using the videoOut
// object </dmu>


// Simple macro for checking the result of a dmu method and printing
// the results if there's ana error.
#define dmuCheck(dmFunc) { \
    int dmuErrID; \
    if ((dmuErrID = (dmFunc)) != DMU_OK) {\
       fprintf(stderr,"*Error:%d:%s\n", dmuErrID, \
	       getErrorString(dmuErrID) ); \
					     /*exit(-1);*/ \
    }\
    else { \
	     /*fprintf(stderr,"No Error\n");*/ \
    } \
 }


int main(int argc, char *argv[]){
  // dmu objects we'll use
  dmuVideoIn    *videoIn;
  dmuGLDisplay  *display;
  dmuVideoOut   *videoOut;

  // other variables we'll need
  fd_set readFDSet;
  int sizeX,sizeY;
  int err;

  // -- Construction --
  // Create our objects.
  videoIn = new dmuVideoIn();
  videoIn->setName("VideoIn object");

  display = new dmuGLDisplay();
  display->setName("Graphics Display");

  videoOut = new dmuVideoOut();
  videoOut->setName("Video Output");

  // -- Wire --
  // Wire up the path (video->display)
  videoIn->addObject(display);
  videoIn->addObject(videoOut);
  // Set up our objects.  We'll print out what we're doing as we go

  // -- Setup --

  //   -- Begin Video Object Setup ----

  // Tells video that we'll use the source that's set on vcp
  fprintf (stderr, "Setting up video path:\n");
  dmuCheck(videoIn->setup(VL_ANY,VL_VIDEO));

  // Tell's the video object that we want YUV pixels
  fprintf (stderr, "Setting up packing: ");
  dmuCheck(videoIn->setPacking(VL_PACKING_YVYU_422_8));
  
  // Get the size of a single frame of video 
  videoIn->getImageDimensions(&sizeX,&sizeY);
  fprintf (stderr, "Video Image Size: %d, %d\n\n",sizeX,sizeY);

  
  dmuCheck(videoOut->setup(VL_ANY));
  dmuCheck(videoOut->setPacking(VL_PACKING_YVYU_422_8));
  dmuCheck(videoOut->setSize(sizeX,sizeY));

  //   -- End Video Setup ---

  //   -- Begin Display Setup --

  // Tells the display object we want YUV pixels.  This is an example
  // of how object set-ups will have to match
  display->setPacking( GL_YCRCB_422_SGIX);

  // Use the video size as the size to display
  display->setImageDimensions( sizeX, sizeY);

  // This will actually create the window to display stuff
  display->setup();
  
  //   -- End Display Setup --

  // -- Initialize --

  // Init only has to be called at the head of the chain.
  fprintf (stderr, "Initializing: \n");
  dmuCheck(videoIn->init());


  // -- Start --

  // Start only has to be called at the head of the chain.  The video
  // transfer has now started.
  fprintf (stderr, "Fire it up!: \n");
  dmuCheck(videoIn->start());

  // -- Begin Handle events loop --
  for (int i = 0; ; i++){
    int maxFD = 0;

    // Setup our file descriptor set
    FD_ZERO(&readFDSet);
    videoIn->setFD(&readFDSet, &maxFD);
    videoOut->setFD(&readFDSet, &maxFD);

    // Wait for something to happen on the video path
    select (maxFD + 1,&readFDSet, NULL, NULL, NULL);
    
    // If something happens, handle the event.  handleEvents
    // automatically propagates down to the display object when it
    // gets a valid event, so we don't have to do anything else!
    err = videoIn->handleEvents(readFDSet);
    if (err == VIDEOIN_TRANSFER_COMPLETE){
      //fprintf (stderr,"got a new frame \n");
    }

    err = videoOut->handleEvents(readFDSet);

  }
  // -- End Handle events loop --

  // On exit, we should destroy the objects in the chain (except we'll 
  // never get here, because we were in an infite loop above :)
  delete (videoIn);
  delete (display);
}
